Attribute VB_Name = "ByteHelperFunctions"
Option Explicit
Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
                                       hpvDest As Any, _
                                       hpvSource As Any, _
                                       ByVal cbCopy As Long)
                                 
Public Function Byte2String(InByte() As Byte, OutString As String, _
                              Optional ResizeString As Boolean) As Boolean
  'Convert entire byte array to string.
  'String must be large enough to hold data.
   
   Dim I As Integer
   On Error GoTo Byte2StringErr
   
   If Not ResizeString Then
      If (Len(OutString) < (Sizeof(InByte))) Then
         Byte2String = False
         Exit Function
      End If
   End If
   
   OutString = StrConv(InByte(), vbUnicode)
   Byte2String = True
   Exit Function
Byte2StringErr:
   Byte2String = False

End Function

Public Function szByte2StringB(InByte() As Byte, OutString As String) As Boolean
  'Convert byte array to string.
  'Stops the conversion at the first null so that any embedded nulls are not included.
   
   Dim I As Integer
   Dim c As Byte
   On Error GoTo szByte2StringBErr
   
   OutString = ""
   For I = LBound(InByte) To UBound(InByte, 1)
      c = InByte(I)
      If InByte(I) = 0 Then Exit For               'stop at null
      OutString = OutString & Chr(c)
   Next I
   szByte2StringB = True
   Exit Function
   
szByte2StringBErr:
   szByte2StringB = False
End Function


Public Function String2Byte(InString As String, OutByte() As Byte) As Boolean
  'vb byte-array / string coercion assumes unicode string
  'so must copy hex data this way...
  'ASSUMPTIONS:
  '   Byte array is zero based
  '   Byte array is sized before being passed
  '   Byte array is padded with nulls if > size of string
   
   Dim I As Integer
   Dim SizeOutByte As Integer
   Dim SizeInString As Integer
   
   SizeOutByte = UBound(OutByte) - LBound(OutByte) + 1
   SizeInString = Len(InString)
   
   'Check for valid sizes
   If (SizeInString = 0) Or _
      (SizeOutByte < SizeInString) Then
      String2Byte = False
      Exit Function
   End If
   
   'Convert the string
   For I = 0 To SizeInString - 1
      OutByte(I) = AscB(Mid(InString, I + 1, 1))
   Next I
   'If byte array > len of string pad
   If SizeOutByte > SizeInString Then           'Pad with Nulls
      For I = SizeInString To UBound(OutByte)
         OutByte(I) = 0
      Next I
   End If
   'ViewByteArray OutByte, "String2Byte"
   String2Byte = True
End Function
Public Function String2Byte2(InString As String, OutByte() As Byte, _
                              Optional AddNull As Boolean) As Boolean
   'This version uses StrConv.  It is faster but will redim the array to the
   'length of the string passed.
   'ASSUMPTIONS:
   '   Byte array is zero based
   '   Byte array will be resized to same length as string
   '   Byte array is resized with one null added to end if AddNull set
   
   Dim I As Integer
   Dim SizeOutByte As Integer
   Dim SizeInString As Integer
   
   'Convert the string
   OutByte = StrConv(InString, vbFromUnicode)
   
   If AddNull Then
      ReDim Preserve OutByte(Sizeof(OutByte)) As Byte
      OutByte(UBound(OutByte)) = 0  'Make null terminated String
   End If
   ViewByteArray OutByte, "String2Byte"
   String2Byte2 = True
End Function
Public Function String2Byte3(InString As String, OutByte() As Byte, _
                              Optional PadWithNulls As Boolean) As Boolean
   'This version uses Kernel32 API to move the szString (converted to
   'ANSI by VB on the API call) data to the array of byte buffer.
   
   'ASSUMPTIONS:
   '   Byte array is zero based
   '   Byte array is sized before being passed
   '
   
   Dim I As Integer
   Dim SizeOutByte As Integer
   Dim SizeInString As Integer
   
   'Check for valid sizes
   SizeOutByte = UBound(OutByte) - LBound(OutByte) + 1
   SizeInString = Len(InString)
   If (SizeInString = 0) Or _
      (SizeOutByte < SizeInString) Then
      String2Byte3 = False
      Exit Function
   End If
   
   'Convert the string
   Call CopyMemory(OutByte(0), ByVal InString, SizeInString)

   
   If PadWithNulls And (SizeOutByte > SizeInString) Then
      For I = SizeInString To UBound(OutByte)
         OutByte(I) = 0  'Make null terminated String
      Next I
   End If
   ViewByteArray OutByte, "String2Byte"
   String2Byte3 = True
End Function

Public Function String2Byte2D(InString As String, OutByte() As Byte, RowIdx As Long) As Boolean
   'Store a String as one element of a 2D byte array for use in block inserts
   '
   'VB byte arrays are layed out in memory opposite of C.  The string would
   'be by column instead of by row so must flip flop the string.
   'ASSUMPTIONS:
   '   Byte array is zero based
   '   Byte array is sized before being passed
   '   Byte array is padded with nulls if > size of string
   
   Dim I As Integer
   Dim SizeOutByte As Integer
   Dim SizeInString As Integer
   
   SizeInString = Len(InString)
   SizeOutByte = UBound(OutByte, 1) - LBound(OutByte, 1) + 1
   If SizeInString > SizeOutByte Then
      String2Byte2D = False
      Exit Function
   End If
   
   'Convert the string
   For I = 0 To SizeInString - 1
      OutByte(I, RowIdx) = AscB(Mid(InString, I + 1, 1))
   Next I
   'If byte array > len of string pad
   If SizeOutByte > SizeInString Then             'Pad with Nulls
      For I = SizeInString To SizeOutByte - 1
         OutByte(I, RowIdx) = 0
      Next I
   End If
   'ViewByteArray OutByte, "String2Byte"
   String2Byte2D = True
End Function

Public Sub ViewByteArray(Data() As Byte, Optional Title As String)
   'Display message box showing hex values of byte array
   
   Dim S As String
   Dim I As Integer
   Dim B As Byte
   Dim S1 As String
   
   On Error GoTo VBANext
   
   S = "Length: " & Str(UBound(Data) - LBound(Data) + 1) & "  Data (in hex):" & vbCrLf
   For I = LBound(Data) To UBound(Data)
      If (I Mod 8) = 0 And I <> 0 Then
         S = S & "  "                           'add extra space every 10th byte
      End If
      If (I Mod 32) = 0 Then S = S & vbCrLf     'CrLf every 32 bytes
      B = Data(I)
      S1 = Hex(Data(I))
      If Len(S1) = 1 Then S1 = "0" & S1         'Format doesn't work too well with hex (0A)
      S = S & S1 & " "
VBANext:
   Next I
   
   MsgBox S, , Title
   
End Sub

Public Function Sizeof(abData() As Byte) As Integer
   'Returns size of an array of byte
   Sizeof = UBound(abData) - LBound(abData) + 1
End Function

Public Sub ViewHex(S As String, Title As String)
'Used to view string as hex...
   Dim I As Integer
   Dim HexString As String
   Dim S1 As String
   
   On Error GoTo NextNum
   HexString = "Length: " & Len(S) & ".  String: " & vbCrLf & S & _
               vbCrLf & vbCrLf & "In hex: " & vbCrLf
   For I = 1 To Len(S)
      'add hex  Str(Asc((Mid(S, I, 1)))) & " "
      S1 = Hex(Asc((Mid(S, I, 1))))
      If Len(S1) = 1 Then S1 = "0" & S1         'pad to same length
      HexString = HexString & S1
      If I Mod 8 = 0 Then _
                     HexString = HexString & " "     'Add extra space every 8
      If I Mod 32 = 0 And I <> 0 Then _
                     HexString = HexString & vbCrLf  'CrLF every 32
      
NextNum:
   Next I
   MsgBox HexString, 0, Title
End Sub


